(* Delphi Dialog
   Info mit Detailanzeige
   ======================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Apr. 2010
   last modified: Nov. 2021
   *)

unit DetailInfoDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms, 
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ImgList,
  System.ImageList;

type
  TInfoIcon = (iiInfo,iiError);

  TDetailInfoDialog = class(TForm)
    btDetails: TBitBtn;
    OKBtn: TBitBtn;
    laInfo: TLabel;
    lbDetails: TListBox;
    ilButton: TImageList;
    pnTop: TPanel;
    imIcon: TImage;
    ilIcon: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure btDetailsClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ShowDetails : boolean;
  public
    { Public-Deklarationen }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    procedure Execute (Apos : TPoint; const Title,Info: string; InfoIcon : TInfoIcon; Details : TStringList);
  end;

procedure ShowDetailedInfo(Apos : TPoint; const Title,Info: string; InfoIcon : TInfoIcon; Details : TStringList);

var
  DetailInfoDialog: TDetailInfoDialog;

implementation

{$R *.dfm}

uses GnuGetText, WinUtils;

procedure TDetailInfoDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TDetailInfoDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then begin
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
    ScaleImageList(ilIcon,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
    ScaleImageList(ilButton,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
    end
  end;
{$EndIf}

procedure TDetailInfoDialog.btDetailsClick(Sender: TObject);
var
  h : integer;
begin
  ShowDetails:=not ShowDetails;
  btDetails.Glyph:=nil;
  h:=0;
  if ShowDetails then begin
    ilButton.GetBitmap(0,btDetails.Glyph);
    h:=94;
    end
  else ilButton.GetBitmap(1,btDetails.Glyph);
  ClientHeight:=pnTop.Height+h;
  end;

procedure TDetailInfoDialog.Execute (APos : TPoint; const Title,Info: string;
                 InfoIcon : TInfoIcon; Details : TStringList);
begin
  AdjustFormPosition(Screen,self,APos);
  ilIcon.GetBitMap(integer(InfoIcon),imIcon.Picture.Bitmap);
  ilButton.GetBitmap(1,btDetails.Glyph);
  ShowDetails:=false;
  ClientHeight:=pnTop.Height;
  if length(Title)>0 then Caption:=Title;
  laInfo.Caption:=Info;
  lbDetails.Items:=Details;
  ShowModal;
  end;

procedure ShowDetailedInfo(APos : TPoint; const Title,Info: string; InfoIcon : TInfoIcon; Details : TStringList);
begin
  if not assigned(DetailInfoDialog) then DetailInfoDialog:=TDetailInfoDialog.Create(Application);
  with DetailInfoDialog do begin
    Execute(APos,Title,Info,InfoIcon,Details);
    Release;
    end;
  DetailInfoDialog:=nil;
  end;

end.
