(* Sprache auswhlen

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jan. 2006
   last modified: Nov. 2021
   *)

unit LanguageDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, LangUtils;

type
  TLanguageDialog = class(TForm)
    lbLanguage: TListBox;
    btbCancel: TBitBtn;
    btbOK: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    Languages        : TLanguageList;
  public
    { Public declarations }
  end;

procedure SelectLanguage;

var
  LanguageDialog: TLanguageDialog;

implementation

{$R *.dfm}

uses GnuGetText, InitProg;

procedure TLanguageDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  Languages:=TLanguageList.Create(PrgPath,LangName);
  Languages.LoadLanguageNames(SelectedLanguage);
  end;

procedure TLanguageDialog.FormDestroy(Sender: TObject);
begin
  Languages.Free;
  end;

procedure SelectLanguage;
var
  i,n,k : integer;
begin
  if not assigned(LanguageDialog) then LanguageDialog:=TLanguageDialog.Create(Application);
  with LanguageDialog do begin
    lbLanguage.Clear; n:=-1;
    with Languages do for i:=1 to Count-1 do begin
      k:=lbLanguage.Items.AddObject(Strings[i],pointer(i));
      if AnsiSameText(SelectedLanguage,LanguageCode[i]) then n:=k;
      end;
    lbLanguage.ItemIndex:=n;
    if ShowModal=mrOK then with lbLanguage do if ItemIndex>=0 then begin
      n:=integer(Items.Objects[ItemIndex]);
      if n>=0 then ChangeLanguage(Languages.LanguageCode[n]);
      end;
    Free;
    end;
  LanguageDialog:=nil;
  end;

initialization
  LanguageDialog:=nil;

end.
