(* Delphi Dialog
   Abfrage der Angaben fr eine Anmeldung (Name + Passwort)
   ========================================================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - Dez. 2004
   last modified: Aug. 2022
   *)

unit LogonDlg;

interface

uses WinApi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, WinUtils;

type
  TLogonDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    SkipBtn: TBitBtn;
    cbName: TComboBox;
    laHinweis: TStaticText;
    PwdEdit: TLabeledEdit;
    Label1: TStaticText;
    procedure NameEditEnter(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private declarations }
    FIniName,FSection : string;
  public
    { Public declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    procedure LoadFromIni(IniName, IniSection : string);
    function Execute(Titel,Hinweis : string;
                     ShowSkip      : boolean;
                     var User,Pwd  : string) : TModalResult;
  end;

function InputUserAccount(Titel,Hinweis : string; ShowSkip : boolean;
                          var User,Pwd  : string) : TModalResult;

var
  LogonDialog: TLogonDialog;

implementation

{$R *.DFM}

uses GnuGetText, FileUtils {$IFDEF ACCESSIBLE}, ShowMessageDlg{$ENDIF};

{ ------------------------------------------------------------------- }
procedure TLogonDialog.LoadFromIni(IniName,IniSection : string);
begin
  FIniName:=IniName; FSection:=IniSection;
  LoadHistory(FIniName,FSection,cbName);
  with cbName do
    if Items.Count=0 then Style:=csSimple else Style:=csDropDown;
  end;

{ ------------------------------------------------------------------- }
procedure TLogonDialog.FormCreate(Sender: TObject);
var
  ok : boolean;
begin
{$IFDEF Trace}
  WriteDebugLog('Create LogonDlg');
{$EndIf}
  TranslateComponent (self,'dialogs');
{$IFDEF ACCESSIBLE}
  if not SystemParametersInfo(SPI_GETSCREENREADER,0,@ok,0) then ok:=false;
  SetTabStops(self,ok);  //only TSTaticText and TEdit/readonly
{$ENDIF}
  cbName.Style:=csSimple;
  FIniName:=''; FSection:=''
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TLogonDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TLogonDialog.FormDestroy(Sender: TObject);
begin
  if (length(FIniName)>0) and (length(FSection)>0) then
    SaveHistory(FIniName,FSection,true,cbName);
  end;

procedure TLogonDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
{$IFDEF ACCESSIBLE}
  if (Key=VK_F11) then begin
    if ActiveControl is TLabeledEdit then with ActiveControl as TLabeledEdit do begin
      if length(Hint)=0 then ShowHintInfo(EditLabel.Caption) else ShowHintInfo(Hint);
      end
    else with ActiveControl do if length(Hint)>0 then ShowHintInfo(Hint);
    end;
{$ENDIF}
  end;

procedure TLogonDialog.NameEditEnter(Sender: TObject);
begin
  PwdEdit.Text:='';
  end;

procedure TLogonDialog.FormActivate(Sender: TObject);
begin
  BringToFront;
  with cbName do if length(Text)=0 then SetFocus
  else PwdEdit.SetFocus;
  end;

{ ------------------------------------------------------------------- }
(* Benutzername und Passwort, Ergebnis: "true" bei "ok" *)
function TLogonDialog.Execute(Titel,Hinweis: string;
                              ShowSkip      : boolean;
                              var User,Pwd  : string) : TModalResult;
var
  mr : TModalResult;
begin
  Caption:=Titel;
  laHinweis.Caption:=Hinweis;
  cbName.Text:=User;
  PwdEdit.Text:='';
  SkipBtn.Visible:=ShowSkip;
  mr:=ShowModal;
  if mr=mrOK then begin
    User:=cbName.Text; Pwd:=PwdEdit.Text;
    AddToHistory(cbName,User);
    end;
  Result:=mr;
  end;

{ ------------------------------------------------------------------- }
(* Benutzername und Passwort, Ergebnis: "true" bei "ok" *)
function InputUserAccount(Titel,Hinweis : string;
                          ShowSkip      : boolean;
                          var User,Pwd  : string) : TModalResult;
begin
  if not assigned(LogonDialog) then LogonDialog:=TLogonDialog.Create(Application);
  with LogonDialog do begin
    Result:=Execute(Titel,Hinweis,ShowSkip,User,Pwd);
    Release;
    end;
  LogonDialog:=nil;
  end;

end.
