(* Delphi-Unit
   Eingabefenster fr mehrzeiligen Text
   ====================================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Mrz. 2008
   last modified: July 2022
   *)

unit MemoDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TMemoDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Memo: TMemo;
    paBottom: TPanel;
    paTop: TPanel;
    Descriptor: TStaticText;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function Execute(APos        : TPoint;
                     const Titel,Desc : string;
                     var AText  : string) : boolean;
  end;

function EditText (APos : TPoint; const ATitle,ADesc : string; var AText : string) : boolean;

var
  MemoDialog: TMemoDialog;

implementation

{$R *.dfm}

uses GnuGetText, WinUtils, {$IFDEF ACCESSIBLE} ShowMessageDlg {$ELSE} MsgDialogs {$ENDIF};

procedure TMemoDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TMemoDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
{$IFDEF ACCESSIBLE}
  if (Key=VK_F11) then begin
    with ActiveControl do if length(Hint)>0 then ShowHintInfo(Hint);
    end;
{$ENDIF}
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TMemoDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

function TMemoDialog.Execute(APos       : TPoint;
                             const Titel,Desc : string;
                             var AText  : string) : boolean;
begin
  AdjustFormPosition(Screen,self,APos);
  Caption:=Titel;
  Descriptor.Caption:=Desc;
  with Memo do begin
    Lines.CommaText:=AText;
    Hint:=Desc;
    end;
  if ShowModal=mrOK then begin
    AText:=Memo.Lines.CommaText;
    Result:=true;
    end
  else Result:=false;
  end;

function EditText (APos : TPoint; const ATitle,ADesc : string; var AText : string) : boolean;
begin
  if not assigned(MemoDialog) then MemoDialog:=TMemoDialog.Create(Application);
  Result:=MemoDialog.Execute(APos,ATitle,ADesc,AText);
  FreeAndNil(MemoDialog);
  end;

end.
