(* Delphi Dialog
   Dateianzeige mit ShellListView
   ==============================

    2009 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Mai 2009
   Vers. 2 - July 2022: define compiler switch "ACCESSIBLE" to make dialog
                        messages accessible to screenreaders
   last modified: July 2022
   *)

unit ShowFilesDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ComCtrls,
  Vcl.Shell.ShellCtrls;

type
  TShowFilesDialog = class(TForm)
    pnTop: TPanel;
    pnBottom: TPanel;
    OKBtn: TBitBtn;
    lvFiles: TShellListView;
    btnDirUp: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnDirUpClick(Sender: TObject);
    procedure lvFilesDblClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private-Deklarationen }
    RootName : string;
    procedure NewColWidths (n1,n2,n3,n4 : integer);
  public
    { Public-Deklarationen }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    procedure Show (const ATitle,Dir  : string);
  end;

procedure ShowDirFiles(const ATitle,Dir  : string);

var
  ShowFilesDialog: TShowFilesDialog;

implementation

{$R *.dfm}

uses System.StrUtils, GnuGetText, ExtSysUtils, PathUtils,FileUtils,
  WinUtils, {$IFDEF ACCESSIBLE} ShowMessageDlg {$ELSE} MsgDialogs {$ENDIF};

{------------------------------------------------------------------- }
procedure TShowFilesDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  NewColWidths(32,12,13,18);
  end;

procedure TShowFilesDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
{$IFDEF ACCESSIBLE}
  if (Key=VK_F11) then begin
    with ActiveControl do if length(Hint)>0 then ShowHintInfo(Hint);
    end;
{$ENDIF}
end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TShowFilesDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TShowFilesDialog.FormResize(Sender: TObject);
var
  n : integer;
begin
  n:=round((MulDiv(ClientWidth,PixelsPerInchOnDesign,Screen.PixelsPerInch)-43*475/75)*75/475);
  NewColWidths(n,12,13,18);
//  lvFiles.SetColWidths([n,10,10,14]);
  end;

procedure TShowFilesDialog.NewColWidths (n1,n2,n3,n4 : integer);
begin
  lvFiles.SetColWidths([MulDiv(n1,Screen.PixelsPerInch,PixelsPerInchOnDesign),
                        MulDiv(n2,Screen.PixelsPerInch,PixelsPerInchOnDesign),
                        MulDiv(n3,Screen.PixelsPerInch,PixelsPerInchOnDesign),
                        MulDiv(n4,Screen.PixelsPerInch,PixelsPerInchOnDesign)]);
  end;

procedure TShowFilesDialog.btnDirUpClick(Sender: TObject);
begin
  with lvFiles do begin
    if not AnsiSameText(RootName,SetDirName(RootFolder.PathName)) then begin
      Back;
      pnTop.Caption:=RootFolder.PathName;
      lvFiles.Hint:=dgettext('dialogs','Directory:')+' '+pnTop.Caption;
      end
    end;
  btnDirUp.Enabled:=not AnsiSameText(RootName,SetDirName(lvFiles.RootFolder.PathName))
  end;

procedure TShowFilesDialog.lvFilesDblClick(Sender: TObject);
begin
  pnTop.Caption:=lvFiles.RootFolder.PathName;
  btnDirUp.Enabled:=not AnsiSameText(RootName,SetDirName(lvFiles.RootFolder.PathName))
  end;

{------------------------------------------------------------------- }
procedure TShowFilesDialog.Show (const ATitle,Dir  : string);
begin
  if length(ATitle)=0 then Caption:=dgettext('dialogs','Show files')
  else Caption:=ATitle;
  if DirectoryExists(Dir) then begin
    RootName:=SetDirName(Dir);
    pnTop.Caption:=Dir;
    lvFiles.Root:=Dir;
    lvFiles.Hint:=dgettext('dialogs','Directory:')+' '+pnTop.Caption;
    btnDirUp.Enabled:=false;
    ShowModal;
    end
  else ErrorDialog(TryFormat(dgettext('dialogs','Directory not found:'+sLineBreak+'%s!'),[Dir]));
  end;

{------------------------------------------------------------------- }
procedure ShowDirFiles(const ATitle,Dir  : string);
begin
  if not assigned(ShowFilesDialog) then ShowFilesDialog:=TShowFilesDialog.Create(Application);
  ShowFilesDialog.Show(ATitle,Dir);
  FreeAndNil(ShowFilesDialog);
  end;

end.
