(*  Delphi Dialog
    Tastaturanpassung fr Syn-Edit
    ==============================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Jun. 2005
   last modified: Nov. 2021
   *)

unit SynEdKeyDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.Menus,
  SynEditKeyCmds, SynEditMiscClasses, Vcl.ComCtrls;

type
  TKeyboardDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    KeyCmdList: TListView;
    Label1: TLabel;
    Label2: TLabel;
    stxCommand: TStaticText;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    procedure KeyCmdListClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
  private
    { Private declarations }
    ks : TSynEditKeyStrokes;
    hkKeystroke : TSynHotKey;
    procedure UpdateKeystrokesList;
  public
    { Public declarations }
    function Execute (var AKeyStrokes : TSynEditKeyStrokes) : boolean;
  end;

var
  KeyboardDialog: TKeyboardDialog;

implementation

{$R *.DFM}

uses GnuGetText;

procedure TKeyboardDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  hkKeystroke := TSynHotKey.Create(self);
  with hkKeystroke do begin
    Parent := KeyboardDialog;
    Left := 270;
    Top := 75;
    Width := 141;
    Height := 19;
    HotKey := 0;
    InvalidKeys := [];
    Modifiers := [];
    TabOrder := 3;
    end;
  end;

procedure TKeyboardDialog.FormDestroy(Sender: TObject);
begin
  hkKeystroke.Free;
  end;

procedure TKeyboardDialog.UpdateKeystrokesList;
var
  i: integer;
begin
  with KeyCmdList,Items do begin
    BeginUpdate;
    try
      Clear;
      for i := 0 to ks.Count - 1 do
        with Add do begin
          Caption := EditorCommandToCodeString(ks[i].Command);
          Data:=pointer(i);
          if ks[i].ShortCut = 0 then SubItems.Add(dgettext('dialogs','<none>'))
          else SubItems.Add(ShortCutToText(ks[i].ShortCut));
        end;
    finally
      EndUpdate;
      if Count>0 then ItemIndex:=0;
    end;
  end;
end;

procedure TKeyboardDialog.KeyCmdListClick(Sender: TObject);
begin
  with KeyCmdList do if ItemIndex>=0 then with Items[ItemIndex] do begin
    stxCommand.Caption:=Caption;
    hkKeystroke.HotKey:=ks.Items[integer(Data)].ShortCut;
    end;
  end;

function TKeyboardDialog.Execute (var AKeyStrokes : TSynEditKeyStrokes) : boolean;
begin
  ks:=AKeyStrokes;
  UpdateKeystrokesList;
  KeyCmdListClick(self);
  if ShowModal=mrOK then begin
    AKeyStrokes:=ks;
    Result:=true;
    end
  else Result:=false;
  end;

procedure TKeyboardDialog.BitBtn1Click(Sender: TObject);
begin
  hkKeystroke.HotKey:=0;
  end;

procedure TKeyboardDialog.BitBtn2Click(Sender: TObject);
begin
  with KeyCmdList do if Items.Count>0 then with Items[ItemIndex] do begin
    SubItems[0]:=ShortCutToText(hkKeystroke.HotKey);
    ks.Items[integer(Data)].ShortCut:=hkKeystroke.HotKey;
    end;
  end;

end.
