(* Delphi dialog
   Edit password with confirmation

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - June 2005
   last modified: Nov. 2021
   *)
   
unit AccountDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TAccountDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    edtRepPwd: TEdit;
    Label1: TLabel;
    edtPwd: TEdit;
    Label2: TLabel;
    edtName: TEdit;
    btnShow: TSpeedButton;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure btnShowClick(Sender: TObject);
  private
    { Private declarations }
    FConf   : boolean;
    FMinLen : integer;
  public
    { Public declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function Execute (const Title : string; var Username,Pwd : string;
                      Confirm : boolean; Show : boolean = false;
                      MinLen : integer = 0) : boolean;
  end;

function InputAccount (const Title : string; var Username,Pwd : string;
                       Confirm : boolean; Show : boolean = false;
                       MinLen : integer = 0) : boolean;

var
  AccountDialog: TAccountDialog;

implementation

{$R *.DFM}

uses GgtDummy, WinUtils;

{------------------------------------------------------------------- }
procedure TAccountDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TAccountDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TAccountDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult=mrOK) and FConf then
    CanClose:=(edtPwd.Text=edtRepPwd.Text) and ((FMinLen=0) or (length(edtPwd.Text)>=FMinLen));
  if not CanClose then begin
    if length(edtPwd.Text)<FMinLen then begin
      ErrorDialog(Caption,Format(dgettext('dialogs','Das Kennwort muss mindestens %u Zeichen lang sein'),[FMinLen]));
      edtPwd.SetFocus;
      end
    else begin
      ErrorDialog(Caption,dgettext('dialogs','Die Kennwortangaben stimmen nicht berein!'));
      edtRepPwd.SetFocus;
      end;
    end;
  end;

procedure TAccountDialog.btnShowClick(Sender: TObject);
var
  c : char;
begin
  if btnShow.Down then c:=#0 else c:='*';
  edtPwd.PasswordChar:=c;
  edtRepPwd.PasswordChar:=c;
  end;

{ ------------------------------------------------------------------- }
(* Passworteingabe, Ergebnis: "true" bei "ok" *)
function TAccountDialog.Execute (const Title    : string;
                                 var Username,Pwd  : string;
                                 Confirm,Show : boolean;
                                 MinLen : integer) : boolean;
begin
  Caption:=Title;
  edtName.Text:=Username;
  edtPwd.Text:=Pwd;
  edtRepPwd.Text:=Pwd;
  FConf:=Confirm;
  FMinLen:=MinLen;
  btnShow.Visible:=Show;
  if FConf then edtRepPwd.Show
  else edtRepPwd.Hide;
  ActiveControl:=edtName;
  if ShowModal=mrOK then begin
    Username:=edtName.Text;
    Pwd:=edtPwd.Text;
    Result:=true;
    end
  else Result:=false;
  end;

{------------------------------------------------------------------- }
function InputAccount (const Title : string; var Username,Pwd : string;
                       Confirm,Show : boolean; MinLen : integer) : boolean;
begin
  if not assigned(AccountDialog) then AccountDialog:=TAccountDialog.create(Application);
  Result:=AccountDialog.execute(Title,Username,Pwd,Confirm,Show,MinLen);
  FreeAndNil(AccountDialog);
  end;

end.
