(* Dialog for FTP proxy connection parameters

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jul. 2007
   last modified: Nov. 2021
   *)

unit FtpProxyDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, NumberEd, FtpUtils;

type
  TFtpProxyDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    edProxy: TEdit;
    edUser: TEdit;
    edProxyPwd: TEdit;
    Label5: TLabel;
    edtPort: TRangeEdit;
    rgMethod: TRadioGroup;
    cbShowPwd: TCheckBox;
    procedure cbShowPwdClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function Execute (var AProxyPar : TFtpProxyParams; ShowPwd : boolean) : boolean;
  end;

function EditFtpProxy (var AProxyPar : TFtpProxyParams; ShowPwd : boolean) : boolean;

var
  FtpProxyDialog: TFtpProxyDialog;

implementation

{$R *.dfm}

uses IdFtp, GgtDummy, WinUtils;

{------------------------------------------------------------------- }
procedure TFtpProxyDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TFtpProxyDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TFtpProxyDialog.cbShowPwdClick(Sender: TObject);
var
  c : char;
begin
  if cbShowPwd.Checked then c:=#0 else c:='*';
  edProxyPwd.PasswordChar:=c;
  end;

{------------------------------------------------------------------- }
function TFtpProxyDialog.Execute (var AProxyPar : TFtpProxyParams; ShowPwd : boolean) : boolean;
begin
  cbShowPwd.Visible:=ShowPwd;
  with AProxyPar do begin
    edProxy.Text:=Server;
    edtPort.Value:=Port;
    edUser.Text:=Username;
    edProxyPwd.Text:=Password;
    with rgMethod do if Mode=fpcmNone then ItemIndex:=0
    else ItemIndex:=integer(Mode)-1;
    ActiveControl:=edProxy;
    if ShowModal=mrOK then begin
      Server:=edProxy.Text;
      Port:=edtPort.Value;
      Username:=edUser.Text;
      Password:=edProxyPwd.Text;
      Mode:=TIdFtpProxyType(rgMethod.ItemIndex+1);
      Result:=true;
      end
    else Result:=false;
    end;
  end;

{------------------------------------------------------------------- }
function EditFtpProxy (var AProxyPar : TFtpProxyParams; ShowPwd : boolean) : boolean;
begin
  if not assigned(FtpProxyDialog) then FtpProxyDialog:=TFtpProxyDialog.Create(Application);
  Result:=FtpProxyDialog.Execute(AProxyPar,ShowPwd);
  FreeAndNil(FtpProxyDialog);
  end;

end.
