(* Delphi-Unit
   Eingabefenster fr mehrzeiligen Text
   ====================================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Mrz. 2008
   last modified: Nov. 2021
   *)

unit MemoDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TMemoDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Descriptor: TLabel;
    Memo: TMemo;
    paBottom: TPanel;
    paTop: TPanel;
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function Execute(APos        : TPoint;
                     const Titel,Desc : string;
                     var AText  : string) : boolean;
  end;

function EditText (APos : TPoint; const ATitle,ADesc : string; var AText : string) : boolean;

var
  MemoDialog: TMemoDialog;

implementation

{$R *.dfm}

uses GgtDummy, WinUtils;

procedure TMemoDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TMemoDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

function TMemoDialog.Execute(APos       : TPoint;
                             const Titel,Desc : string;
                             var AText  : string) : boolean;
begin
  AdjustFormPosition(Screen,self,APos);
  Caption:=Titel;
  Descriptor.Caption:=Desc;
  Memo.Lines.CommaText:=AText;
  if ShowModal=mrOK then begin
    AText:=Memo.Lines.CommaText;
    Result:=true;
    end
  else Result:=false;
  end;

function EditText (APos : TPoint; const ATitle,ADesc : string; var AText : string) : boolean;
begin
  if not assigned(MemoDialog) then MemoDialog:=TMemoDialog.Create(Application);
  Result:=MemoDialog.Execute(APos,ATitle,ADesc,AText);
  FreeAndNil(MemoDialog);
  end;

end.
